import 'dart:async';

import 'package:analyzer/dart/element/element.dart';
import 'package:build/build.dart';
import 'package:source_gen/source_gen.dart';

/// Similar to AnnotationGenerator but allows custom filter implementation
abstract class FilterGenerator extends Generator {
  BuilderOptions options;
  FilterGenerator(this.options);

  bool elementFilter(Element element);
  Future<String> generateForElement(Element element);

  @override
  generate(LibraryReader library, BuildStep buildStep) async {
    var futures = library.allElements
        .where(elementFilter)
        .map((e) async => await generateForElement(e))
        .toList();
    var result = await Future.wait(futures);
    return result.join('\n\n');
  }
}
